@@ -1,6 +1,9 @@
-use crate::err::{ArgumentError, Result};
-use crate::pat::{ParseIndex, ParserPattern};
-use crate::Ustr;
+use ustr::Ustr;
+
+use crate::err::Error;
+use crate::err::Result;
+use crate::pat::ParseIndex;
+use crate::pat::ParserPattern;
 
 /// Parse the input command line item with given prefixs, return an [`DataKeeper`].
 ///
@@ -24,35 +27,36 @@ use crate::Ustr;
 /// # Example
 ///
 /// ```rust
-/// use aopt::arg::parser;
+/// use aopt::arg::parse_argument;
 /// use ustr::Ustr;
+/// use aopt::gstr;
 /// use aopt::err::Result;
 ///
 /// fn main() -> Result<()> {
-///     let prefix = &[Ustr::from("--"), Ustr::from("-")];
+///     let prefix = &[gstr("--"), gstr("-")];
 ///
 ///     {// parse option with value
-///         let dk = parser::parse_argument(Ustr::from("--foo=32"), prefix)?;
+///         let dk = parse_argument(gstr("--foo=32"), prefix)?;
 ///
-///         assert_eq!(dk.prefix, Some(Ustr::from("--")));
-///         assert_eq!(dk.name, Some(Ustr::from("foo")));
-///         assert_eq!(dk.value, Some(Ustr::from("32")));
+///         assert_eq!(dk.prefix, Some(gstr("--")));
+///         assert_eq!(dk.name, Some(gstr("foo")));
+///         assert_eq!(dk.value, Some(gstr("32")));
 ///         assert_eq!(dk.disable, false);
 ///     }
 ///     {// parse boolean option
-///         let dk = parser::parse_argument(Ustr::from("--/bar"), prefix)?;
+///         let dk = parse_argument(gstr("--/bar"), prefix)?;
 ///
-///         assert_eq!(dk.prefix, Some(Ustr::from("--")));
-///         assert_eq!(dk.name, Some(Ustr::from("bar")));
+///         assert_eq!(dk.prefix, Some(gstr("--")));
+///         assert_eq!(dk.name, Some(gstr("bar")));
 ///         assert_eq!(dk.value, None);
 ///         assert_eq!(dk.disable, true);
 ///     }
 ///     {// parse other string
-///         let dk = parser::parse_argument(Ustr::from("-=bar"), prefix)?;
+///         let dk = parse_argument(gstr("-=bar"), prefix)?;
 ///
-///         assert_eq!(dk.prefix, Some(Ustr::from("-")));
+///         assert_eq!(dk.prefix, Some(gstr("-")));
 ///         assert_eq!(dk.name, None);
-///         assert_eq!(dk.value, Some(Ustr::from("bar")));
+///         assert_eq!(dk.value, Some(gstr("bar")));
 ///         assert_eq!(dk.disable, false);
 ///     }
 ///     Ok(())
@@ -75,7 +79,7 @@ pub fn parse_argument(pattern: Ustr, prefix: &[Ustr]) -> Result<DataKeeper> {
         return Ok(data_keeper);
     }
     error!(?pattern, ?prefix, ?index, "parsing argument failed");
-    Err(ArgumentError::ParsingFailed(pattern.get_pattern().to_owned()).into())
+    Err(Error::arg_parsing_failed(pattern.get_pattern()))
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
@@ -196,12 +200,11 @@ impl State {
                                 ?pattern,
                                 "accessing string [{}, {}) failed", start, name_end
                             );
-                            return Err(ArgumentError::PatternAccessFailed(
-                                pattern.get_pattern().to_owned(),
+                            return Err(Error::arg_pattern_out_of_range(
+                                pattern.get_pattern(),
                                 start,
                                 name_end,
-                            )
-                            .into());
+                            ));
                         }
                         break;
                     }
@@ -225,18 +228,15 @@ impl State {
                             index.get(),
                             index.len()
                         );
-                        return Err(ArgumentError::PatternAccessFailed(
-                            pattern.get_pattern().to_owned(),
+                        return Err(Error::arg_pattern_out_of_range(
+                            pattern.get_pattern(),
                             index.get(),
                             index.len(),
-                        )
-                        .into());
+                        ));
                     }
                 } else {
                     error!(?pattern, "syntax error! require an value after '='.");
-                    return Err(
-                        ArgumentError::MissingValue(pattern.get_pattern().to_owned()).into(),
-                    );
+                    return Err(Error::arg_missing_value(pattern.get_pattern()));
                 }
             }
             Self::End => {
@@ -252,8 +252,10 @@ impl State {
 
 #[cfg(test)]
 mod test {
+    use ustr::Ustr;
+
     use crate::arg::parser::parse_argument;
-    use crate::Ustr;
+    use crate::gstr;
 
     #[test]
     fn test_for_input_parser() {
@@ -280,10 +282,10 @@ mod test {
                 ("-foo=", None),
             ];
 
-            let prefixs = vec![Ustr::from("--"), Ustr::from("-"), Ustr::from("")];
+            let prefixs = vec![gstr("--"), gstr("-"), gstr("")];
 
             for case in test_cases.iter() {
-                try_to_verify_one_task(Ustr::from(case.0), &prefixs, case.1);
+                try_to_verify_one_task(gstr(case.0), &prefixs, case.1);
             }
         }
         {
@@ -309,10 +311,10 @@ mod test {
                 ("-foo=", None),
             ];
 
-            let prefixs = vec![Ustr::from("--"), Ustr::from("-")];
+            let prefixs = vec![gstr("--"), gstr("-")];
 
             for case in test_cases.iter() {
-                try_to_verify_one_task(Ustr::from(case.0), &prefixs, case.1);
+                try_to_verify_one_task(gstr(case.0), &prefixs, case.1);
             }
         }
     }
@@ -331,7 +333,7 @@ mod test {
                 panic!("----> {:?} {:?}", except, &dk);
             }
 
-            let default = Ustr::from("");
+            let default = gstr("");
 
             if let Some(except) = except {
                 assert_eq!(
