@@ -58,13 +58,13 @@ impl Display for ErrorStr {
 
 #[derive(Debug, Error)]
 pub enum Error {
-    #[error("Argument error: `{0}`")]
+    #[error("Argument error: {0}")]
     FromArgumentError(#[from] ArgumentError),
 
-    #[error("Construct error: `{0}`")]
+    #[error("Construct error: {0}")]
     FromConstrutError(#[from] ConstructError),
 
-    #[error("Special error: `{0}`")]
+    #[error("Special error: {0}")]
     FromSpecialError(#[from] SpecialError),
 
     #[error("{0}")]
@@ -129,6 +129,11 @@ impl Error {
         ConstructError::InvalidRetValueOfCallback(t.into()).into()
     }
 
+    /// Create ConstructError::CanNotInsertPOSIfCMDExists error
+    pub fn opt_can_not_insert_pos() -> Self {
+        ConstructError::CanNotInsertPOSIfCMDExists.into()
+    }
+
     /// Create ConstructError::MissingOptionType error
     pub fn opt_missing_type<T: Into<ErrorStr>>(t: T) -> Self {
         ConstructError::MissingOptionType(t.into()).into()
@@ -200,8 +205,13 @@ impl Error {
     }
 
     /// Create SpecialError::POSForceRequired error
-    pub fn sp_pos_force_require<T: Into<ErrorStr>>(p: u64, t: T) -> Self {
-        SpecialError::POSForceRequired(p, t.into()).into()
+    pub fn sp_pos_force_require<T: Into<ErrorStr>>(t: T) -> Self {
+        SpecialError::POSForceRequired(t.into()).into()
+    }
+
+    /// Create SpecialError::CMDForceRequired error
+    pub fn sp_cmd_force_require<T: Into<ErrorStr>>(t: T) -> Self {
+        SpecialError::CMDForceRequired(t.into()).into()
     }
 
     /// Create SpecialError::InvalidOptionName error
@@ -286,6 +296,9 @@ pub enum ConstructError {
 
     #[error("Invalid callback return value type: `{0}`")]
     InvalidRetValueOfCallback(ErrorStr),
+
+    #[error("Can not have force required POS if CMD exists")]
+    CanNotInsertPOSIfCMDExists,
 }
 
 /// Special error using for parser.
@@ -303,8 +316,11 @@ pub enum SpecialError {
     #[error("Invalid value for option `{0}`")]
     InvalidArgumentForOption(ErrorStr),
 
-    #[error("POS `{1} @{0}` is force required")]
-    POSForceRequired(u64, ErrorStr),
+    #[error("POS `{0}` is force required")]
+    POSForceRequired(ErrorStr),
+
+    #[error("CMD `{0}` is force required")]
+    CMDForceRequired(ErrorStr),
 
     #[error("Invalid option name: `{0}`")]
     InvalidOptionName(ErrorStr),
