@@ -75,9 +75,9 @@
 //!     Ok(())
 //! }
 //! ```
-//! 
+//!
 //! The above code output:
-//! 
+//!
 //! ```txt
 //! APP: test-app
 //! OPTION: `depth` -> : Int(42)
@@ -88,125 +88,125 @@
 //!
 //! ## Setup
 //!
-//! Add the following to your `Cargo.toml` file:
+//! Add following to your `Cargo.toml` file:
 //!
 //! ```toml
 //! [dependencies]
 //! aopt = "0.5"
 //! ```
-//! 
+//!
 //! ### Enable `sync` feature
-//! 
-//! If you want the utils of current crate implement [`Send`] and [`Sync`], you can enable `sync` feature. 
-//! 
+//!
+//! If you want the utils of current crate implement [`Send`] and [`Sync`], you can enable `sync` feature.
+//!
 //! ```toml
 //! [dependencies]
 //! aopt = { version = "0.5", features = [ "sync" ] }
 //! ```
-//! 
+//!
 //! ## Feature
-//! 
+//!
 //! In following example, the type of `parser` is `Parser<SimpleSet, DefaultService, ForwardPolicy>`.
-//! 
+//!
 //! ### Option
-//! 
+//!
 //! See the option create string help here: [`parse_option_str`](crate::opt::parse_option_str).
-//! 
-//! The implementation of option is [`Opt`](crate::opt::Opt). 
+//!
+//! The implementation of option is [`Opt`](crate::opt::Opt).
 //! It is matched prefix and name of option argument.
-//! 
-//! * Type support 
-//! 
-//! Common type options are built-in support. 
+//!
+//! * Type support
+//!
+//! Common type options are built-in support.
 //! Such as [`b`](crate::opt::opt::BoolOpt) and [`s`](crate::opt::opt::StrOpt).
-//! With typed option support, you can keep typed value, 
-//! and customize the behavior when the user set it. 
+//! With typed option support, you can keep typed value,
+//! and customize the behavior when the user set it.
 //! And you can add new type if necessary.
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
 //! parser.add_opt("--foo=b")?.commit()?; // add option `foo` with type `b`
 //! parser.add_opt("--bar=s")?.commit()?; // add option `bar` with type `s`
 //! ```
-//! 
+//!
 //! #### Built-in type
-//! 
+//!
 //! [`b`](crate::opt::opt::BoolOpt) with value type [`bool`].
-//! 
-//! [`i`](crate::opt::opt::IntOpt) with value type [`i64`]. 
-//! 
+//!
+//! [`i`](crate::opt::opt::IntOpt) with value type [`i64`].
+//!
 //! [`u`](crate::opt::opt::UintOpt) with value type [`u64`].
-//! 
+//!
 //! [`f`](crate::opt::opt::FltOpt) with value type [`f64`].
-//! 
+//!
 //! [`s`](crate::opt::opt::StrOpt) with value type [`String`].
-//! 
+//!
 //! [`a`](crate::opt::opt::ArrayOpt) with value type [`Vec`].
-//! 
+//!
 //! #### Any type option
-//! 
+//!
 //! [`PathOpt`](crate::opt::opt::PathOpt) is an exmaple option keep [`PathBuf`](std::path::PathBuf) inside.
-//! 
+//!
 //! * Callback support
-//! 
+//!
 //! The option can have associate [`OptCallback`](crate::opt::OptCallback).
 //! The [`Parser`] will call it if user set the option.
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
-//! parser.add_opt_cb("--foo=s", 
+//! parser.add_opt_cb("--foo=s",
 //!     simple_opt_cb!(|uid, set, value| {
 //!     assert_eq!(value, OptValue::from("bar"));
 //!     Ok(Some(value))
 //! }))?.commit()?
 //! // user can set the option `foo` like: `app.exe --foo=bar`
 //! ```
-//! 
+//!
 //! * Prefix support
-//! 
+//!
 //! You can customize the prefix.
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
 //! parser.add_prefix("+".into()); // add support for prefix `+`
 //! parser.add_opt("+F=a")?.commit()?;
 //! // user can set the option `F` like: `app.exe +F foo +F bar`
 //! ```
-//! 
+//!
 //! * Alias support
-//! 
+//!
 //! You can have one or more alias.
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
 //! parser.add_opt("--foo=s")?.add_alias("-f")?.commit()?;
 //! // user can set the option `foo` like: `app.exe -f value`
 //! ```
-//! 
+//!
 //! * Value support
-//! 
+//!
 //! You can keep a type value in the option, and it can have a default value.
-//! 
+//!
 //! ### non-option
-//! 
+//!
 //! In implementation side, [`NonOpt`](crate::opt::NonOpt) is based on [`Opt`](crate::opt::Opt).
 //! Unlike the option matched with name and prefix of option argument.
 //! The non-option is matched with [`OptIndex`](crate::opt::OptIndex)(based on 1) or name of non-option argument.
-//! 
+//!
 //! * `p`: [`PosOpt`](crate::opt::nonopt::PosOpt)
-//! 
+//!
 //! The `PosOpt` will match the index, and call the callback with type [`PosFn`](crate::opt::PosFn).
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
 //! parser.add_opt("--foo=b")?.commit()?; // add option `foo` with type `b`
 //! parser.add_opt("--bar=s")?.commit()?; // add option `bar` with type `s`
-//! parser.add_opt("arg=p@1", 
+//! parser.add_opt("arg=p@1",
 //!     simple_pos_cb!(|uid, set, arg, index, value| {
 //!     // will get `foo` inside value here
 //!     assert_eq!(value, OptValue::from("foo"));
@@ -214,107 +214,107 @@
 //! }))?.commit()?;
 //! // user can set the option like: `app.exe --foo --bar value foo`
 //! ```
-//! 
+//!
 //! * `c`: [`CmdOpt`](crate::opt::nonopt::CmdOpt)
-//! 
+//!
 //! The `CmdOpt` is an specify [`PosOpt`](crate::opt::nonopt::PosOpt) with [`Forward`](crate::opt::OptIndex::Forward)(1).
 //! It will match the name, and call the callback with type [`MainFn`](crate::opt::MainFn).
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
 //! parser.add_opt("--foo=b")?.commit()?; // add option `foo` with type `b`
 //! parser.add_opt("--bar=s")?.commit()?; // add option `bar` with type `s`
-//! parser.add_opt("show=c", 
+//! parser.add_opt("show=c",
 //!     simple_main_cb!(|uid, set, args, value| {
 //!     assert_eq!(args, &["show", "foo"]);
 //!     Ok(Some(value))
 //! }))?.commit()?;
 //! // user can set the option like: `app.exe show --foo --bar value foo`
-//! 
+//!
 //! ```
 //! * `m`: [`MainOpt`](crate::opt::nonopt::MainOpt)
-//! 
+//!
 //! The `MainOpt` will always be called with the callback type [`MainFn`](crate::opt::MainFn).
-//! 
+//!
 //! #### Example
-//! 
+//!
 //! ```ignore
 //! parser.add_opt("--foo=b")?.commit()?; // add option `foo` with type `b`
 //! parser.add_opt("--bar=s")?.commit()?; // add option `bar` with type `s`
-//! parser.add_opt("default_main=m", 
+//! parser.add_opt("default_main=m",
 //!     simple_main_cb!(|uid, set, args, value| {
 //!     assert_eq!(args, &["foo", "bar"]);
 //!     Ok(Some(value))
 //! }))?.commit()?;
 //! // user can set the option like: `app.exe --foo --bar value foo bar`
 //! ```
-//! 
+//!
 //! ### Policy
-//! 
+//!
 //! [`Policy`] is responsible for analyze the input command argument,
 //! and match it with option in given [`Set`].
-//! 
+//!
 //! #### ForwardPolicy
-//! 
+//!
 //! Parsing step of [`ForwardPolicy`](crate::parser::ForwardPolicy):
-//! 
+//!
 //! * Go through the command line arguments.
-//! 
+//!
 //!     If the argument like an option.
-//! 
+//!
 //!     - Generate and process [`OptMatcher`](crate::proc::OptMatcher) of [`PSEqualWithValue`](crate::parser::ParserState::PSEqualWithValue)、[`PSArgument`](crate::parser::ParserState::PSArgument)、[`PSBoolean`](crate::parser::ParserState::PSBoolean)、[`PSMultipleOption`](crate::parser::ParserState::PSMultipleOption) and [`PSEmbeddedValue`](crate::parser::ParserState::PSEmbeddedValue). Invoke the callback if the any option matched.
-//! 
+//!
 //!     - Return an Err if the option not matched and the strict flag is true.
-//! 
+//!
 //!     Otherwise, add it to `NOA`(non-option argument) array.
-//! 
+//!
 //! * Generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonCmd`](crate::parser::ParserState::PSNonCmd).
-//! 
+//!
 //! * Go through the `NOA` array, and generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonPos`](crate::parser::ParserState::PSNonPos).
-//! 
+//!
 //! * Generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonMain`](crate::parser::ParserState::PSNonMain).
-//! 
+//!
 //! #### PrePolicy
-//! 
+//!
 //! Parsing step of [`PrePolicy`](crate::parser::PrePolicy):
-//! 
+//!
 //! * Go through the command line arguments.
-//! 
+//!
 //!     If the argument like an option.
-//! 
+//!
 //!     - Generate and process [`OptMatcher`](crate::proc::OptMatcher) of [`PSEqualWithValue`](crate::parser::ParserState::PSEqualWithValue)、[`PSArgument`](crate::parser::ParserState::PSArgument)、[`PSBoolean`](crate::parser::ParserState::PSBoolean)、[`PSMultipleOption`](crate::parser::ParserState::PSMultipleOption) and [`PSEmbeddedValue`](crate::parser::ParserState::PSEmbeddedValue), call the callback if the any option matched.
-//! 
+//!
 //!     - __Add it to `NOA` array if the option not matched.__
-//! 
+//!
 //!     Otherwise, add it to `NOA`(non-option argument) array.
-//! 
+//!
 //! * Generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonCmd`](crate::parser::ParserState::PSNonCmd).
-//! 
+//!
 //! * Go through the `NOA` array, and generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonPos`](crate::parser::ParserState::PSNonPos).
-//! 
+//!
 //! * Generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonMain`](crate::parser::ParserState::PSNonMain).
-//! 
+//!
 //! #### DelayPolicy
-//! 
+//!
 //! Parsing step of [`DelayPolicy`](crate::parser::DelayPolicy):
-//! 
+//!
 //! * Go through the command line arguments.
-//! 
+//!
 //!     If the argument like an option.
-//! 
+//!
 //!     - Generate and process [`OptMatcher`](crate::proc::OptMatcher) of [`PSDelayEqualWithValue`](crate::parser::ParserState::PSDelayEqualWithValue)、[`PSDelayArgument`](crate::parser::ParserState::PSDelayArgument)、[`PSDelayBoolean`](crate::parser::ParserState::PSDelayBoolean)、[`PSDelayMultipleOption`](crate::parser::ParserState::PSDelayMultipleOption) and [`PSDelayEmbeddedValue`](crate::parser::ParserState::PSDelayEmbeddedValue). __Add the callback invoke context to [`ValueKeeper`](crate::parser::ValueKeeper) array if any option matched__.
-//! 
+//!
 //!     - Return an Err if the option not matched and the strict flag is true.
-//! 
+//!
 //!     Otherwise, add it to `NOA`(non-option argument) array.
-//! 
+//!
 //! * Generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonCmd`](crate::parser::ParserState::PSNonCmd).
-//! 
+//!
 //! * Go through the `NOA` array, and generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonPos`](crate::parser::ParserState::PSNonPos).
-//! 
+//!
 //! * __Invoke the callback that saved in [`ValueKeeper`](crate::parser::ValueKeeper) array.__
-//! 
+//!
 //! * Generate and process [`NonOptMatcher`](crate::proc::NonOptMatcher) of [`PSNonMain`](crate::parser::ParserState::PSNonMain).
 pub mod app;
 pub mod arg;
@@ -355,7 +355,7 @@ where
     I: Into<String>,
     ITER: Iterator<Item = I>,
     S: Set,
-    SS: Service,
+    SS: Service<S>,
 {
     let args: Vec<String> = iter.map(|v| v.into()).collect();
     let count = parsers.len();
@@ -391,7 +391,7 @@ where
     I: Into<String>,
     ITER: Iterator<Item = I>,
     S: Set,
-    SS: Service,
+    SS: Service<S>,
     P: Policy<S, SS>,
 {
     let args: Vec<String> = iter.map(|v| v.into()).collect();
@@ -626,6 +626,7 @@ pub mod prelude {
     pub use crate::parser::PreParser;
     pub use crate::parser::PrePolicy;
     pub use crate::parser::Service;
+    pub use crate::parser::SimpleService;
     pub use crate::proc::Info;
     pub use crate::proc::Matcher;
     pub use crate::proc::NonOptMatcher;
@@ -635,6 +636,7 @@ pub mod prelude {
     pub use crate::set::OptionSet;
     pub use crate::set::PrefixSet;
     pub use crate::set::Set;
+    pub use crate::set::SetIndex;
     pub use crate::set::SimpleSet;
     pub use crate::simple_main_cb;
     pub use crate::simple_main_mut_cb;
