@@ -346,8 +346,8 @@ impl Value {
         *self = Self::Null;
     }
 
-    /// Append string if current value is [`Value::Array`].
-    pub fn app_str(&mut self, string: String) -> &mut Self {
+    /// Add string if current value is [`Value::Array`].
+    pub fn add_str(&mut self, string: String) -> &mut Self {
         match self {
             Self::Array(v) => {
                 v.push(string);
@@ -356,6 +356,110 @@ impl Value {
         }
         self
     }
+
+    /// Append string if current value is [`Value::Array`].
+    pub fn app_str(&mut self, values: &[String]) -> &mut Self {
+        match self {
+            Self::Array(v) => {
+                for value in values {
+                    v.push(value.clone());
+                }
+            }
+            _ => {}
+        }
+        self
+    }
+
+    /// Return a new value contains inner strings and other's strings if type is [`Value::Array`].
+    ///
+    /// It will take the ownership of strings of current value.
+    pub fn merge(&mut self, other: &Self) -> Option<Self> {
+        match self {
+            Self::Array(values) => {
+                let mut ret = vec![];
+
+                if let Some(ov) = other.as_vec() {
+                    for value in values {
+                        ret.push(std::mem::take(value))
+                    }
+                    for item in ov {
+                        ret.push(item.clone());
+                    }
+                    Some(Self::from(ret))
+                } else {
+                    None
+                }
+            }
+            _ => None,
+        }
+    }
+
+    /// Return a new value contains inner strings and other's strings if type is [`Value::Array`].
+    ///
+    /// It will take the ownership of strings of current value.
+    /// And also will take ownership of strings in other value.
+    pub fn merge_mut(&mut self, other: &mut Self) -> Option<Self> {
+        match self {
+            Self::Array(values) => {
+                let mut ret = vec![];
+
+                if let Some(ov) = other.as_vec_mut() {
+                    for value in values {
+                        ret.push(std::mem::take(value))
+                    }
+                    for value in ov {
+                        ret.push(std::mem::take(value));
+                    }
+                    Some(Self::from(ret))
+                } else {
+                    None
+                }
+            }
+            _ => None,
+        }
+    }
+
+    pub fn take_int(&mut self) -> Option<i64> {
+        match self {
+            Self::Int(v) => Some(std::mem::take(v)),
+            _ => None,
+        }
+    }
+
+    pub fn take_uint(&mut self) -> Option<u64> {
+        match self {
+            Self::Uint(v) => Some(std::mem::take(v)),
+            _ => None,
+        }
+    }
+
+    pub fn take_flt(&mut self) -> Option<f64> {
+        match self {
+            Self::Flt(v) => Some(std::mem::take(v)),
+            _ => None,
+        }
+    }
+
+    pub fn take_str(&mut self) -> Option<String> {
+        match self {
+            Self::Str(v) => Some(std::mem::take(v)),
+            _ => None,
+        }
+    }
+
+    pub fn take_bool(&mut self) -> Option<bool> {
+        match self {
+            Self::Bool(v) => Some(std::mem::take(v)),
+            _ => None,
+        }
+    }
+
+    pub fn take_vec(&mut self) -> Option<Vec<String>> {
+        match self {
+            Self::Array(v) => Some(std::mem::take(v)),
+            _ => None,
+        }
+    }
 }
 
 impl Clone for Value {
