@@ -24,9 +24,11 @@ pub use delay_policy::DelayPolicy;
 pub use forward_policy::ForwardPolicy;
 pub use pre_policy::PrePolicy;
 pub use service::CallbackStore;
-pub use service::DefaultService;
+pub use service::SimpleService;
 pub use state::ParserState;
 
+pub type DefaultService = SimpleService<SimpleSet>;
+
 /// Default [`Parser`] that using [`ForwardPolicy`].
 pub type ForwardParser = Parser<SimpleSet, DefaultService, ForwardPolicy>;
 
@@ -50,7 +52,7 @@ pub struct ValueKeeper {
 /// #[derive(Debug)]
 /// pub struct EmptyPolicy;
 ///
-/// impl<S: Set, SS: Service> Policy<S, SS> for EmptyPolicy {
+/// impl<S: Set, SS: Service<S>> Policy<S, SS> for EmptyPolicy {
 ///     fn parse(
 ///         &mut self,
 ///         set: &mut S,
@@ -62,7 +64,7 @@ pub struct ValueKeeper {
 ///     }
 /// }
 /// ```
-pub trait Policy<S: Set, SS: Service>: Debug {
+pub trait Policy<S: Set, SS: Service<S>>: Debug {
     fn parse(
         &mut self,
         set: &mut S,
@@ -72,7 +74,7 @@ pub trait Policy<S: Set, SS: Service>: Debug {
 }
 
 /// [`Service`] provide common service using for [`Policy`].
-pub trait Service {
+pub trait Service<S: Set> {
     /// Generate M base on [`Argument`] and [`ParserState`].
     fn gen_opt<M: Matcher + Default>(
         &self,
@@ -93,7 +95,7 @@ pub trait Service {
     ///
     /// The `invoke` should be false if caller don't want invoke callback when
     /// [`Opt`](crate::opt::Opt) matched.
-    fn matching<M: Matcher + Default, S: Set>(
+    fn matching<M: Matcher + Default>(
         &mut self,
         matcher: &mut M,
         set: &mut S,
@@ -101,19 +103,22 @@ pub trait Service {
     ) -> Result<Vec<ValueKeeper>>;
 
     /// Checking if the `set` data valid.
-    fn pre_check<S: Set>(&self, set: &S) -> Result<bool>;
+    fn pre_check(&self, set: &S) -> Result<bool>;
 
     /// Checking if the `set` data valid.
-    fn opt_check<S: Set>(&self, set: &S) -> Result<bool>;
+    fn opt_check(&self, set: &S) -> Result<bool>;
 
     /// Checking if the `set` data valid.
-    fn nonopt_check<S: Set>(&self, set: &S) -> Result<bool>;
+    fn pos_check(&self, set: &S) -> Result<bool>;
 
     /// Checking if the `set` data valid.
-    fn post_check<S: Set>(&self, set: &S) -> Result<bool>;
+    fn cmd_check(&self, set: &S) -> Result<bool>;
+
+    /// Checking if the `set` data valid.
+    fn post_check(&self, set: &S) -> Result<bool>;
 
     /// Invoke callback connected with given [`Opt`](crate::opt::Opt).
-    fn invoke<S: Set>(
+    fn invoke(
         &mut self,
         uid: Uid,
         set: &mut S,
@@ -122,7 +127,7 @@ pub trait Service {
     ) -> Result<Option<OptValue>>;
 
     /// Return the callback map reference.
-    fn get_callback(&self) -> &CallbackStore;
+    fn get_callback(&self) -> &CallbackStore<S>;
 
     /// Return the subscriber info vector reference.
     fn get_subscriber_info<I: 'static + Info>(&self) -> &Vec<Box<dyn Info>>;
@@ -131,7 +136,7 @@ pub trait Service {
     fn get_noa(&self) -> &Vec<Ustr>;
 
     /// Return the callback map mutable reference.
-    fn get_callback_mut(&mut self) -> &mut CallbackStore;
+    fn get_callback_mut(&mut self) -> &mut CallbackStore<S>;
 
     /// Return the subscriber info vector mutable reference.
     fn get_subscriber_info_mut(&mut self) -> &mut Vec<Box<dyn Info>>;
@@ -155,7 +160,7 @@ pub trait Service {
 ///     #[derive(Debug, Default)]
 ///     pub struct EmptyPolicy(i64);
 ///
-///     impl<S: Set, SS: Service> Policy<S, SS> for EmptyPolicy {
+///     impl<S: Set, SS: Service<S>> Policy<S, SS> for EmptyPolicy {
 ///         fn parse(
 ///             &mut self,
 ///             set: &mut S,
@@ -185,7 +190,7 @@ pub trait Service {
 pub struct Parser<S, SS, P>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
     P: Policy<S, SS>,
 {
     policy: P,
@@ -196,7 +201,7 @@ where
 impl<S, SS, P> Default for Parser<S, SS, P>
 where
     S: Set + Default,
-    SS: Service + Default,
+    SS: Service<S> + Default,
     P: Policy<S, SS> + Default,
 {
     fn default() -> Self {
@@ -211,7 +216,7 @@ where
 impl<S, SS, P> Deref for Parser<S, SS, P>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
     P: Policy<S, SS>,
 {
     type Target = S;
@@ -224,7 +229,7 @@ where
 impl<S, SS, P> DerefMut for Parser<S, SS, P>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
     P: Policy<S, SS>,
 {
     fn deref_mut(&mut self) -> &mut Self::Target {
@@ -235,7 +240,7 @@ where
 impl<S, SS, P> Parser<S, SS, P>
 where
     S: Set + Default,
-    SS: Service + Default,
+    SS: Service<S> + Default,
     P: Policy<S, SS>,
 {
     /// Initialize the [`Parser`] with specify [`Policy`] and the
@@ -251,7 +256,7 @@ where
     ///     #[derive(Debug)]
     ///     pub struct EmptyPolicy;
     ///
-    ///     impl<S: Set, SS: Service> Policy<S, SS> for EmptyPolicy {
+    ///     impl<S: Set, SS: Service<S>> Policy<S, SS> for EmptyPolicy {
     ///         fn parse(
     ///             &mut self,
     ///             set: &mut S,
@@ -278,7 +283,7 @@ where
 impl<S, SS, P> Parser<S, SS, P>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
     P: Policy<S, SS>,
 {
     pub fn new(set: S, service: SS, policy: P) -> Self {
@@ -297,6 +302,11 @@ where
         &mut self.policy
     }
 
+    pub fn set_policy(&mut self, policy: P) -> &mut Self {
+        self.policy = policy;
+        self
+    }
+
     pub fn get_service(&self) -> &SS {
         &self.service
     }
@@ -305,6 +315,11 @@ where
         &mut self.service
     }
 
+    pub fn set_service(&mut self, service: SS) -> &mut Self {
+        self.service = service;
+        self
+    }
+
     pub fn get_set(&self) -> &S {
         &self.set
     }
@@ -313,11 +328,16 @@ where
         &mut self.set
     }
 
+    pub fn set_set(&mut self, set: S) -> &mut Self {
+        self.set = set;
+        self
+    }
+
     // extern the add_opt function, attach callback to option
     pub fn add_opt_cb(
         &mut self,
         opt_str: &str,
-        callback: OptCallback,
+        callback: OptCallback<S>,
     ) -> Result<CallbackCommit<'_, '_, S, SS>> {
         let info = CreateInfo::parse(gstr(opt_str), self.get_prefix())?;
 
@@ -330,7 +350,7 @@ where
         ))
     }
 
-    pub fn add_callback(&mut self, uid: Uid, callback: OptCallback) {
+    pub fn add_callback(&mut self, uid: Uid, callback: OptCallback<S>) {
         self.get_service_mut()
             .get_callback_mut()
             .add_callback(uid, callback);
@@ -343,6 +363,21 @@ where
 
         policy.parse(set, service, iter)
     }
+
+    pub fn reset(&mut self) {
+        self.service.reset();
+        self.set.reset();
+    }
+}
+
+cfg_if::cfg_if! {
+    if #[cfg(feature = "sync")] {
+        unsafe impl<S, SS, P> Send for Parser<S, SS, P>
+            where S: Set, SS: Service<S>, P: Policy<S, SS> { }
+
+        unsafe impl<S, SS, P> Sync for Parser<S, SS, P>
+            where S: Set, SS: Service<S>, P: Policy<S, SS> { }
+    }
 }
 
 /// DynParser manage the [`Set`], [`Service`] and [`Policy`].
@@ -357,7 +392,7 @@ where
 ///     #[derive(Debug, Default)]
 ///     pub struct EmptyPolicy;
 ///
-///     impl<S: Set, SS: Service> Policy<S, SS> for EmptyPolicy {
+///     impl<S: Set, SS: Service<S>> Policy<S, SS> for EmptyPolicy {
 ///         fn parse(
 ///             &mut self,
 ///             set: &mut S,
@@ -372,7 +407,7 @@ where
 ///     #[derive(Debug, Default)]
 ///     pub struct ConsumeIterPolicy;
 ///
-///     impl<S: Set, SS: Service> Policy<S, SS> for ConsumeIterPolicy {
+///     impl<S: Set, SS: Service<S>> Policy<S, SS> for ConsumeIterPolicy {
 ///         fn parse(
 ///             &mut self,
 ///             set: &mut S,
@@ -405,7 +440,7 @@ where
 pub struct DynParser<S, SS>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
 {
     policy: Box<dyn Policy<S, SS>>,
     service: SS,
@@ -415,7 +450,7 @@ where
 impl<S, SS> Deref for DynParser<S, SS>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
 {
     type Target = S;
 
@@ -427,7 +462,7 @@ where
 impl<S, SS> DerefMut for DynParser<S, SS>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
 {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.set
@@ -437,7 +472,7 @@ where
 impl<S, SS> DynParser<S, SS>
 where
     S: Set + Default,
-    SS: Service + Default,
+    SS: Service<S> + Default,
 {
     pub fn new_policy<P: Policy<S, SS> + 'static>(policy: P) -> Self {
         Self {
@@ -451,7 +486,7 @@ where
 impl<S, SS> DynParser<S, SS>
 where
     S: Set,
-    SS: Service,
+    SS: Service<S>,
 {
     pub fn new<P: Policy<S, SS> + 'static>(set: S, service: SS, policy: P) -> Self {
         Self {
@@ -469,6 +504,11 @@ where
         self.policy.as_mut()
     }
 
+    pub fn set_policy<P: Policy<S, SS> + 'static>(&mut self, policy: P) -> &mut Self {
+        self.policy = Box::new(policy);
+        self
+    }
+
     pub fn get_service(&self) -> &SS {
         &self.service
     }
@@ -477,6 +517,11 @@ where
         &mut self.service
     }
 
+    pub fn set_service(&mut self, service: SS) -> &mut Self {
+        self.service = service;
+        self
+    }
+
     pub fn get_set(&self) -> &S {
         &self.set
     }
@@ -485,11 +530,16 @@ where
         &mut self.set
     }
 
+    pub fn set_set(&mut self, set: S) -> &mut Self {
+        self.set = set;
+        self
+    }
+
     // extern the add_opt function, attach callback to option
     pub fn add_opt_cb(
         &mut self,
         opt_str: &str,
-        callback: OptCallback,
+        callback: OptCallback<S>,
     ) -> Result<CallbackCommit<'_, '_, S, SS>> {
         let info = CreateInfo::parse(gstr(opt_str), self.get_prefix())?;
 
@@ -502,7 +552,7 @@ where
         ))
     }
 
-    pub fn add_callback(&mut self, uid: Uid, callback: OptCallback) {
+    pub fn add_callback(&mut self, uid: Uid, callback: OptCallback<S>) {
         self.get_service_mut()
             .get_callback_mut()
             .add_callback(uid, callback);
@@ -515,12 +565,17 @@ where
 
         policy.parse(set, service, iter)
     }
+
+    pub fn reset(&mut self) {
+        self.service.reset();
+        self.set.reset();
+    }
 }
 
 impl<S, SS, P> From<Parser<S, SS, P>> for DynParser<S, SS>
 where
     S: Set + Default,
-    SS: Service + Default,
+    SS: Service<S> + Default,
     P: Policy<S, SS> + Default + 'static,
 {
     fn from(mut parser: Parser<S, SS, P>) -> Self {
@@ -537,7 +592,7 @@ where
 impl<'a, S, SS, P> From<&'a mut Parser<S, SS, P>> for DynParser<S, SS>
 where
     S: Set + Default,
-    SS: Service + Default,
+    SS: Service<S> + Default,
     P: Policy<S, SS> + Default + 'static,
 {
     fn from(parser: &'a mut Parser<S, SS, P>) -> Self {
@@ -550,3 +605,13 @@ where
         DynParser::new(set, service, policy)
     }
 }
+
+cfg_if::cfg_if! {
+    if #[cfg(feature = "sync")] {
+        unsafe impl<S, SS> Send for DynParser<S, SS>
+            where S: Set, SS: Service<S> { }
+
+        unsafe impl<S, SS> Sync for DynParser<S, SS>
+            where S: Set, SS: Service<S> { }
+    }
+}
