@@ -43,7 +43,7 @@ impl ForwardPolicy {
     }
 }
 
-impl<S: Set, SS: Service> Policy<S, SS> for ForwardPolicy {
+impl<S: Set, SS: Service<S>> Policy<S, SS> for ForwardPolicy {
     fn parse(
         &mut self,
         set: &mut S,
@@ -137,6 +137,8 @@ impl<S: Set, SS: Service> Policy<S, SS> for ForwardPolicy {
                 service.matching(&mut proc, set, true)?;
             }
 
+            service.cmd_check(set)?;
+
             let gen_style = ParserState::PSNonPos;
 
             info!("start process {:?} ...", &gen_style);
@@ -154,7 +156,7 @@ impl<S: Set, SS: Service> Policy<S, SS> for ForwardPolicy {
 
         // check pos and cmd
         info!("do nonopt check");
-        service.nonopt_check(set)?;
+        service.pos_check(set)?;
 
         let gen_style = ParserState::PSNonMain;
 
@@ -193,7 +195,7 @@ mod test {
         testing_cases.append(&mut long_prefix_opt_testcases());
         testing_cases.append(&mut shorting_prefix_opt_testcases());
 
-        let mut parser = Parser::<SimpleSet, DefaultService, ForwardPolicy>::default();
+        let mut parser = ForwardParser::default();
 
         parser.get_policy_mut().set_strict(false);
 
@@ -261,7 +263,7 @@ mod test {
         testing_cases.append(&mut long_prefix_opt_testcases());
         testing_cases.append(&mut shorting_prefix_opt_testcases());
 
-        let mut parser = Parser::<SimpleSet, DefaultService, ForwardPolicy>::default();
+        let mut parser = ForwardParser::default();
 
         for testing_case in testing_cases.iter_mut() {
             testing_case.add_test(&mut parser)?;
@@ -325,7 +327,7 @@ mod test {
         testing_cases.append(&mut long_prefix_opt_testcases());
         testing_cases.append(&mut shorting_prefix_opt_testcases());
 
-        let mut parser = Parser::<SimpleSet, DefaultService, ForwardPolicy>::default();
+        let mut parser = ForwardParser::default();
 
         for testing_case in testing_cases.iter_mut() {
             testing_case.add_test(&mut parser)?;
