@@ -6,6 +6,7 @@ use super::state::ParserState;
 use super::Service;
 use super::ValueKeeper;
 use crate::opt::OptCallback;
+use crate::opt::OptIndex;
 use crate::opt::OptValue;
 use crate::opt::Style;
 use crate::proc::Info;
@@ -19,36 +20,40 @@ use ustr::Ustr;
 
 /// Simple wrapper of `HashMap<Uid, OptCallback>`.
 #[derive(Debug, Default)]
-pub struct CallbackStore(pub HashMap<Uid, OptCallback>);
+pub struct CallbackStore<S: Set>(pub HashMap<Uid, OptCallback<S>>);
 
-impl Deref for CallbackStore {
-    type Target = HashMap<Uid, OptCallback>;
+impl<S: Set> Deref for CallbackStore<S> {
+    type Target = HashMap<Uid, OptCallback<S>>;
 
     fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
 
-impl DerefMut for CallbackStore {
+impl<S: Set> DerefMut for CallbackStore<S> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.0
     }
 }
 
-impl CallbackStore {
-    pub fn add_callback(&mut self, uid: Uid, cb: OptCallback) {
+impl<S: Set> CallbackStore<S> {
+    pub fn new() -> Self {
+        Self(HashMap::default())
+    }
+
+    pub fn add_callback(&mut self, uid: Uid, cb: OptCallback<S>) {
         self.0.insert(uid, cb);
     }
 
-    pub fn get_callback(&self, uid: &Uid) -> Option<&OptCallback> {
+    pub fn get_callback(&self, uid: &Uid) -> Option<&OptCallback<S>> {
         self.0.get(uid)
     }
 
-    pub fn get_callback_mut(&mut self, uid: &Uid) -> Option<&mut OptCallback> {
+    pub fn get_callback_mut(&mut self, uid: &Uid) -> Option<&mut OptCallback<S>> {
         self.0.get_mut(uid)
     }
 
-    pub fn for_each(&self, f: impl Fn(&Uid, &OptCallback) -> Result<bool>) -> Result<bool> {
+    pub fn for_each(&self, f: impl Fn(&Uid, &OptCallback<S>) -> Result<bool>) -> Result<bool> {
         for (uid, cb) in self.0.iter() {
             f(uid, cb)?;
         }
@@ -57,18 +62,32 @@ impl CallbackStore {
 }
 
 /// Simple implementation of [`Service`].
-#[derive(Debug, Default)]
-pub struct DefaultService {
+#[derive(Debug)]
+pub struct SimpleService<S: Set> {
     noa: Vec<Ustr>,
 
     subscriber_info: Vec<Box<dyn Info>>,
 
-    callback_store: CallbackStore,
+    callback_store: CallbackStore<S>,
 }
 
-impl DefaultService {
+impl<S: Set + Default> Default for SimpleService<S> {
+    fn default() -> Self {
+        Self {
+            callback_store: CallbackStore::new(),
+            noa: Vec::default(),
+            subscriber_info: Vec::default(),
+        }
+    }
+}
+
+impl<S: Set> SimpleService<S> {
     pub fn new() -> Self {
-        Self { ..Self::default() }
+        Self {
+            callback_store: CallbackStore::new(),
+            noa: Vec::default(),
+            subscriber_info: Vec::default(),
+        }
     }
 
     pub fn register<I: 'static + Info>(&mut self, info: I) -> &mut Self {
@@ -76,7 +95,7 @@ impl DefaultService {
         self
     }
 
-    pub fn matching_nonopt<M: Matcher, S: Set>(
+    pub fn matching_nonopt<M: Matcher>(
         &mut self,
         matcher: &mut M,
         set: &mut S,
@@ -92,7 +111,7 @@ impl DefaultService {
 
             if let Some(ctx) = ctx {
                 if ctx.is_matched() {
-                    let opt = set[uid].as_mut();
+                    let opt = set.get_opt(uid).unwrap();
                     let invoke_callback = opt.is_need_invoke();
                     let mut value = ctx.take_value();
 
@@ -117,12 +136,12 @@ impl DefaultService {
                         // reborrow the opt avoid the compiler error
                         // reset the matcher, we need match all the NonOpt
                         debug!(?value, "get callback return value");
-                        set[uid].as_mut().set_invoke(false);
+                        set.get_opt_mut(uid).unwrap().set_invoke(false);
                         matcher.reset();
                     }
 
                     // set the value after invoke
-                    set[uid].as_mut().set_callback_ret(value)?;
+                    set.get_opt_mut(uid).unwrap().set_callback_ret(value)?;
                 }
             }
         }
@@ -132,7 +151,7 @@ impl DefaultService {
         Ok(vec![])
     }
 
-    pub fn matching_opt<M: Matcher, S: Set>(
+    pub fn matching_opt<M: Matcher>(
         &mut self,
         msg: &mut M,
         set: &mut S,
@@ -148,7 +167,8 @@ impl DefaultService {
 
             if let Some(ctx) = ctx {
                 if ctx.is_matched() {
-                    let opt = set[uid].as_mut();
+                    // just unwrap it, the uid must exists
+                    let opt = set.get_opt_mut(uid).unwrap();
                     let invoke_callback = opt.is_need_invoke();
                     let value = ctx.take_value();
 
@@ -175,7 +195,7 @@ impl DefaultService {
                 } else {
                     Some(value)
                 };
-                set[id].as_mut().set_callback_ret(ret_value)?;
+                set.get_opt_mut(id).unwrap().set_callback_ret(ret_value)?;
             }
             return Ok(vec![]);
         }
@@ -186,7 +206,7 @@ impl DefaultService {
     }
 }
 
-impl Service for DefaultService {
+impl<S: Set> Service<S> for SimpleService<S> {
     fn gen_opt<M: Matcher + Default>(
         &self,
         arg: &crate::arg::Argument,
@@ -205,7 +225,7 @@ impl Service for DefaultService {
         Ok(style.gen_nonopt(noa, total as u64, current as u64)?)
     }
 
-    fn matching<M: Matcher + Default, S: Set>(
+    fn matching<M: Matcher + Default>(
         &mut self,
         matcher: &mut M,
         set: &mut S,
@@ -214,7 +234,9 @@ impl Service for DefaultService {
         Ok(self.process(matcher, set, invoke)?)
     }
 
-    fn pre_check<S: Set>(&self, set: &S) -> Result<bool> {
+    /// Check the [Callback](crate::opt::callback::Callback)'s type is matched with option [`CallbackType`](crate::opt::callback::CallbackType).
+    /// Check CMD and force required POS@1 are not exists same time.
+    fn pre_check(&self, set: &S) -> Result<bool> {
         self.callback_store.for_each(|uid, cb| {
             if let Some(opt) = set.get_opt(*uid) {
                 if !opt.is_accept_callback_type(cb.to_callback_type()) {
@@ -229,10 +251,29 @@ impl Service for DefaultService {
                 warn!(%uid, "callback has unknow option uid");
                 Ok(true)
             }
-        })
+        })?;
+        let has_cmd = set.opt_iter().any(|v| v.match_style(Style::Cmd));
+
+        const MAX_INDEX: u64 = u64::MAX;
+
+        if has_cmd {
+            for opt in set.opt_iter() {
+                if opt.as_ref().match_style(Style::Pos) {
+                    if let Some(index) = opt.as_ref().get_index() {
+                        let index = index.calc_index(MAX_INDEX, 1).unwrap_or(MAX_INDEX);
+                        if index == 1 && !opt.as_ref().get_optional() {
+                            // if we have cmd, can not have force required POS @1
+                            return Err(Error::opt_can_not_insert_pos());
+                        }
+                    }
+                }
+            }
+        }
+        Ok(true)
     }
 
-    fn opt_check<S: Set>(&self, set: &S) -> Result<bool> {
+    /// Check if the option is valid.
+    fn opt_check(&self, set: &S) -> Result<bool> {
         for opt in set.opt_iter() {
             if opt.as_ref().match_style(Style::Boolean)
                 || opt.as_ref().match_style(Style::Argument)
@@ -244,103 +285,109 @@ impl Service for DefaultService {
         Ok(true)
     }
 
-    fn nonopt_check<S: Set>(&self, set: &S) -> Result<bool> {
-        const MAX_INDEX: u64 = u64::MAX;
-
+    /// Check if the POS is valid.
+    /// For which POS is have certainty position, POS has same position are replaceble even it is force reuqired.
+    /// For which POS is have uncertainty position, it must be set if it is force reuqired.
+    fn pos_check(&self, set: &S) -> Result<bool> {
+        // for POS has certainty position, POS has same position are replaceble even it is force reuqired.
         let mut index_map: HashMap<u64, Vec<Uid>> = HashMap::new();
+        // for POS has uncertainty position, it must be set if it is force reuqired
+        let mut float_vec: Vec<Uid> = vec![];
 
         for opt in set.opt_iter() {
-            if opt.as_ref().match_style(Style::Pos)
-                || opt.as_ref().match_style(Style::Cmd)
-                || opt.as_ref().match_style(Style::Main)
-            {
-                if let Some(index) = opt.as_ref().get_index() {
-                    let index = index.calc_index(MAX_INDEX, 1).unwrap_or(MAX_INDEX);
-                    let entry = index_map.entry(index).or_insert(vec![]);
-
-                    entry.push(opt.as_ref().get_uid());
+            if opt.as_ref().match_style(Style::Pos) {
+                if let Some(index) = opt.get_index() {
+                    match index {
+                        OptIndex::Forward(_) | OptIndex::Backward(_) => {
+                            if let Some(index) = index.calc_index(u64::MAX, 1) {
+                                let entry = index_map.entry(index).or_insert(vec![]);
+                                entry.push(opt.get_uid());
+                            }
+                        }
+                        OptIndex::List(v) => {
+                            for index in v {
+                                let entry = index_map.entry(*index).or_insert(vec![]);
+                                entry.push(opt.get_uid());
+                            }
+                        }
+                        OptIndex::Except(_)
+                        | OptIndex::Greater(_)
+                        | OptIndex::Less(_)
+                        | OptIndex::AnyWhere => {
+                            float_vec.push(opt.get_uid());
+                        }
+                        OptIndex::Null => {}
+                    }
                 }
             }
         }
-
-        trace!(?index_map, "non-opt check information");
-
         let mut names = vec![];
 
+        trace!(?index_map, ?float_vec, "pos check information");
         for (index, uids) in index_map.iter() {
-            let valid;
+            // if any of POS is force required, then it must set by user
+            let mut pos_valid = true;
 
-            // <cmd1> <cmd2> <pos3> [pos4] [pos5]
-            // any of thing at position 1
-            if index == &1 || index == &0 {
-                let mut cmd_count = 0;
-                let mut cmd_valid = false;
-                let mut pos_valid = true;
-                let mut force_valid = false;
+            for uid in uids {
+                let opt = set.get_opt(*uid).unwrap();
+                let opt_valid = opt.check().is_ok();
 
-                for uid in uids {
-                    let opt = set.get_opt(*uid).unwrap();
-
-                    if opt.match_style(Style::Cmd) {
-                        cmd_count += 1;
-                        // set the cmd will valid the check
-                        // if any of cmd is valid, break out
-                        cmd_valid = cmd_valid || opt.check().is_ok();
-                        if cmd_valid {
-                            break;
-                        }
-                        names.push(opt.get_hint().to_owned());
-                    } else if opt.match_style(Style::Pos) {
-                        let opt_valid = opt.check().is_ok();
-
-                        pos_valid = pos_valid && opt_valid;
-                        if opt_valid && !opt.get_optional() {
-                            force_valid = true;
-                            names.push(opt.get_hint().to_owned());
-                        }
-                    }
+                pos_valid = pos_valid && opt_valid;
+                if !opt_valid {
+                    names.push(opt.get_hint().to_owned());
                 }
-
-                debug!(%cmd_valid, %pos_valid, %force_valid, "in default nonopt-check");
-
-                // if we have CMD, then the CMD must be set or any POS is set
-                // if all nonopt @1 are POS, it's normally like @2..
-                if cmd_count > 0 {
-                    valid = cmd_valid || (pos_valid && force_valid);
-                } else {
-                    valid = pos_valid;
-                }
-            } else {
-                // <pos1> [pos2] [pos3] [pos4] [pos5]
-                // if any of POS is force required, then it must set by user
-                let mut pos_valid = true;
-
-                for uid in uids {
-                    let opt = set.get_opt(*uid).unwrap();
-                    let opt_valid = opt.check().is_ok();
-
-                    pos_valid = pos_valid && opt_valid;
-                    if !opt_valid {
-                        names.push(opt.get_hint().to_owned());
-                    }
-                }
-                debug!(%pos_valid, "in default nonopt-check");
-                valid = pos_valid;
             }
-            if !valid {
-                return Err(Error::sp_pos_force_require(*index, names.join(" | ")));
+            debug!(%pos_valid, "in default pos check @ pos {}", index);
+            if !pos_valid {
+                return Err(Error::sp_pos_force_require(names.join(" | ")));
             }
             names.clear();
         }
-
+        if float_vec.len() > 0 {
+            float_vec
+                .iter()
+                .filter(|&uid| set.get_opt(*uid).unwrap().check().is_err())
+                .for_each(|&uid| {
+                    names.push(set.get_opt(uid).unwrap().get_hint().to_owned());
+                });
+            if names.len() > 0 {
+                debug!(?names, "in default float pos check @ pos");
+                return Err(Error::sp_pos_force_require(names.join(" | ")));
+            }
+        }
         Ok(true)
     }
 
-    fn post_check<S: Set>(&self, _set: &S) -> Result<bool> {
+    fn cmd_check(&self, set: &S) -> Result<bool> {
+        let mut names = vec![];
+        let mut valid = false;
+
+        for opt in set.opt_iter() {
+            if opt.as_ref().match_style(Style::Cmd) {
+                valid = valid || opt.check().is_ok();
+                if valid {
+                    break;
+                } else {
+                    names.push(opt.get_hint().to_owned());
+                }
+            }
+        }
+        if !valid && names.len() > 0 {
+            return Err(Error::sp_cmd_force_require(names.join(" | ")));
+        }
         Ok(true)
     }
 
-    fn invoke<S: Set>(
+    fn post_check(&self, set: &S) -> Result<bool> {
+        for opt in set.opt_iter() {
+            if opt.as_ref().match_style(Style::Main) {
+                opt.check()?;
+            }
+        }
+        Ok(true)
+    }
+
+    fn invoke(
         &mut self,
         uid: Uid,
         set: &mut S,
@@ -375,7 +422,7 @@ impl Service for DefaultService {
         }
     }
 
-    fn get_callback(&self) -> &CallbackStore {
+    fn get_callback(&self) -> &CallbackStore<S> {
         &self.callback_store
     }
 
@@ -387,7 +434,7 @@ impl Service for DefaultService {
         &self.noa
     }
 
-    fn get_callback_mut(&mut self) -> &mut CallbackStore {
+    fn get_callback_mut(&mut self) -> &mut CallbackStore<S> {
         &mut self.callback_store
     }
 
@@ -406,13 +453,8 @@ impl Service for DefaultService {
     }
 }
 
-impl<M: Matcher> Proc<M> for DefaultService {
-    fn process<S: Set>(
-        &mut self,
-        msg: &mut M,
-        set: &mut S,
-        invoke: bool,
-    ) -> Result<Vec<ValueKeeper>> {
+impl<S: Set, M: Matcher> Proc<S, M> for SimpleService<S> {
+    fn process(&mut self, msg: &mut M, set: &mut S, invoke: bool) -> Result<Vec<ValueKeeper>> {
         match msg.get_style() {
             Style::Boolean | Style::Argument | Style::Multiple => {
                 self.matching_opt(msg, set, invoke)
