@@ -1,9 +1,12 @@
 use crate::arg::Argument;
-use crate::ctx::{Context, NonOptContext, OptContext};
-use crate::err::{ArgumentError, Error, Result};
+use crate::ctx::Context;
+use crate::ctx::NonOptContext;
+use crate::ctx::OptContext;
+use crate::err::Error;
+use crate::err::Result;
 use crate::opt::Style;
 use crate::proc::Matcher;
-use crate::Ustr;
+use ustr::Ustr;
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum ParserState {
@@ -46,7 +49,7 @@ pub enum ParserState {
 
 impl ParserState {
     fn gen_unwrap_error(name: &str) -> Error {
-        ArgumentError::UnwrapError(name.to_owned()).into()
+        Error::arg_unwrap_value_failed(name)
     }
 
     fn do_unwrap(name: &str, value: &Option<Ustr>) -> Result<Ustr> {
